--[[
	Created by Elthrìka-Stromrage (EU)
]]--
	 
local cfg, IntN = ...

function HasInterrupt()
	has = false
	for i,spellID in pairs(IntN.playerInterrupts) do
		s,d,e = GetSpellCooldown(spellID)
		has = has or (s==0)
	end
	return has
end

function shouldEnable(unitID)
	if(cfg.enableOnTargetOnly and unitID ~= "target") then return false end
	if(cfg.enableInCombatOnly and not UnitAffectingCombat("player")) then return false end
	_,itype,dif = GetInstanceInfo()
	playerNumber = GetNumGroupMembers()
	if(playerNumber==0 and not cfg.enableInSolo) then return false end
	if(playerNumber>5 and not cfg.enableInRaid) then return false end
	if(playerNumber>0 and playerNumber<=5 and not cfg.enableInGroup) then return false end
	return true
end

function Warn(unitID, name, spellName, duration, interruptible)
	hasInterrupt = HasInterrupt()
	
	if(interruptible and UnitCanAttack("player", unitID) and hasInterrupt) then
		
		if(duration > 0) then
			_,h = PlaySound("raidwarning", "SFX")
			IntN.soundsplaying[#(IntN.soundsplaying)+1] = h
		
			IntN.fs:SetText("Interrupt " .. unitName)

			C_Timer.After( duration, function() IntN.fs:Hide() end )
			IntN.fs:Show()
		end
		
	end
end

function ToggleMsgTest()
	if(IntN.fs:IsShown()) then
		IntN.fs:Hide();
	else
		IntN.fs:Show();
	end
end

function OnEvent(fun, args)
	unitID = args[1]
	
	if(not shouldEnable(unitID)) then return end
	
	spellName, _, _, _, startTime, endTime, _, _, notInterruptible = UnitCastingInfo(unitID)
	unitName = GetUnitName(unitID, true)
	
	d = 0
	if(startTime and endTime) then
		d = (endTime-startTime)/1000
	end
	
	Warn(unitID, unitName, spellName, d, not notInterruptible)
end

function OnEvent_UNIT_SPELLCAST_START(self, event, ...)
	local args = { n = select("#", ...), ... }
	OnEvent(UnitCastingInfo, args)
end

function OnEvent_UNIT_SPELLCAST_CHANNEL_START(self, event, ...)
	local args = { n = select("#", ...), ... }
	OnEvent(UnitChannelInfo, args)
end

function UpdateInterruptsList(self, event, ...)
	
	local args = { n = select("#", ...), ... }
	
	--print("(Re)Loaded because of event: " .. event)
	
	local _,playerClass = UnitClass("player")
	local spec = GetSpecialization()
	local interrupts = {}
	local id1 = cfg.interruptSpellIdBySpec[playerClass]
	local id2 = 0
	if(spec ~= nil) then
		id2 = cfg.interruptSpellIdBySpec[playerClass..spec]
	end
	if(id1 ~= nil and id1 > 0) then
		if(IsSpellKnown(id1, false) or IsSpellKnown(id1, true)) then
			interrupts[#interrupts+1] = id1
		end
	end
	if(id2 ~= nil and id2 > 0) then
		if(IsSpellKnown(id2, false) or IsSpellKnown(id2, true)) then
			interrupts[#interrupts+1] = id2
		end
	end
	
	--for i,h in pairs(interrupts) do
	--	n = GetSpellInfo(h)
	--	print("InterruptNotifier registerd Interrupt: " .. n)
	--end
	
	IntN.playerInterrupts = interrupts
end

function OnEvent_Stop(self, event, ...)
	local args = { n = select("#", ...), ... }
	if(args.n > 0 and args[1] == "target") then
		fs:Hide()
	end
end

function PlayerLogin(self, event, ...)
	SetupCFG()
end


function Setup()	
	IntN = {}
	SetupCFG()
	
	cfg.events = {
		["PLAYER_ENTERING_WORLD"] = UpdateInterruptsList,
		["UNIT_PET"] = UpdateInterruptsList,
		["UNIT_SPELLCAST_CHANNEL_START"] = OnEvent_UNIT_SPELLCAST_CHANNEL_START,
		["UNIT_SPELLCAST_START"] = OnEvent_UNIT_SPELLCAST_START,
		["UNIT_SPELLCAST_CHANNEL_STOP"] = OnEvent_Stop,
		["UNIT_SPELLCAST_STOP"] = OnEvent_Stop,
		["PLAYER_LOGIN"] = PlayerLogin
	}

	f = CreateFrame("Frame")
	
	for k,v in pairs(cfg.events) do
		f:RegisterEvent(k)
	end
	
	f:SetScript("OnEvent", 	function(self, event, ...)
								if(cfg.events[event]) then
									cfg.events[event](self, event, ...)
								end
							end 
				)

	fs = f:CreateFontString()

	fs:SetSize(500,48)
	fs:SetJustifyH("CENTER")
	fs:SetJustifyV("CENTER")
	fs:SetFont("Fonts\\FRIZQT__.TTF", 32, "OUTLINE")
	fs:SetTextColor(1,0.8,0,1)
	fs:SetPoint("CENTER", nil, 0, -40)

	IntN.frame = f
	IntN.fs = fs
	
	IntN.soundsplaying = {}
	IntN.playerInterrupts = {}
	
	
	UpdateInterruptsList(nil, "SETUP_DIRECT_CALL")
	
	for k,v in pairs(IntN.playerInterrupts) do
		name = GetSpellInfo(v)
	end
	
end

function SetupCFG()
	cfg = {}
	
	if(type(IntN_Settings) ~= "table") then
		IntN_Settings = {
			enableInRaid = true,
			enableInSolo = false,
			enableInGroup = true,
			enableInCombatOnly = true,
			enableOnTargetOnly = true
		}
	else
		for k,v in pairs(IntN_Settings) do
			cfg[k] = v
		end
	end
	
	cfg.interruptSpellIdBySpec = {
		["DEATHKNIGHT"] = 47528, --All specs
		["DEMONHUNTER"] = 183752, --All specs
		["DRUID1"] = 78675, --Balance
		["DRUID2"] = 106839, --Feral
		["DRUID3"] = 106839, --Guardian
		["DRUID4"] = -1, --Restoration
		["HUNTER1"] = 147362, --Beast Mastery
		["HUNTER2"] = 147362, --Marksmanship
		["HUNTER3"] = 187707, --Survival
		["MAGE"] =  2139, --All specs
		["MONK1"] = 116705, --Brewmaster
		["MONK2"] = -1, --Mistweaver
		["MONK3"] = 116705, --Wind Walker
		["PALADIN1"] = -1, --Holy
		["PALADIN2"] = 96231, --Protection
		["PALADIN3"] = 96231, --Retribution
		["PRIEST1"] = -1, --Discipline
		["PRIEST2"] = -1, --Holy
		["PRIEST3"] = 15487, --Shadow
		["ROGUE"] = 1766, --All specs
		["SHAMAN"] = 57994, --All specs
		["WARLOCK"] = 171138, --All specs (Doomguard)
		["WARLOCK1"] = 119910, --Affliction (Fel Hunter)
		["WARRIOR"] = 6552, --All specs
		["BLOODELF"] = 50613 --Racial
	}
	
	cfg.frame = CreateFrame("Frame")
	cfg.frame.name = "InterruptNotifier"
	cfg.frame.okay = SaveSettings
	
	cfg.frame.title = CreateFrame("Frame", "IntNTitle", cfg.frame)
	cfg.frame.title:SetPoint("TOPLEFT", 5, -5)
	cfg.frame.title:SetScale(2.0)
	cfg.frame.title:SetWidth(150)
	cfg.frame.title:SetHeight(24)
	cfg.frame.title:Show()

	cfg.frame.titleFS = cfg.frame.title:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	cfg.frame.titleFS:SetText("Interrupt Notifier - Options")
	cfg.frame.titleFS:SetPoint("TOPLEFT", 0, 0)
	
	--RaidCB
	cfg.frame.RaidCB = CreateFrame("CheckButton", "RaidCB", cfg.frame, "InterfaceOptionsCheckButtonTemplate")
	cfg.frame.RaidCB:SetChecked(cfg.enableInRaid)
	_G[cfg.frame.RaidCB:GetName().."Text"]:SetText("Enable in Raid")
	cfg.frame.RaidCB:SetPoint("TOPLEFT", cfg.frame.title, "BOTTOMLEFT", 0, -5)
	cfg.frame.RaidCB:Show()
	
	--SoloCB
	cfg.frame.SoloCB = CreateFrame("CheckButton", "SoloCB", cfg.frame, "InterfaceOptionsCheckButtonTemplate")
	cfg.frame.SoloCB:SetChecked(cfg.enableInSolo)
	_G[cfg.frame.SoloCB:GetName().."Text"]:SetText("Enable when Solo")
	cfg.frame.SoloCB:SetPoint("TOPLEFT", cfg.frame.RaidCB, "BOTTOMLEFT", 0, -5)
	cfg.frame.SoloCB:Show()
	
	--GroupCB
	cfg.frame.GroupCB = CreateFrame("CheckButton", "GroupCB", cfg.frame, "InterfaceOptionsCheckButtonTemplate")
	cfg.frame.GroupCB:SetChecked(cfg.enableInGroup)
	_G[cfg.frame.GroupCB:GetName().."Text"]:SetText("Enable in Group")
	cfg.frame.GroupCB:SetPoint("TOPLEFT", cfg.frame.SoloCB, "BOTTOMLEFT", 0, -5)
	cfg.frame.GroupCB:Show()

	--TargetOnlyCB
	cfg.frame.TargetOnlyCB = CreateFrame("CheckButton", "TargetOnlyCB", cfg.frame, "InterfaceOptionsCheckButtonTemplate")
	cfg.frame.TargetOnlyCB:SetChecked(cfg.enableOnTargetOnly)
	_G[cfg.frame.TargetOnlyCB:GetName().."Text"]:SetText("Enable on Target Only")
	cfg.frame.TargetOnlyCB:SetPoint("TOPLEFT", cfg.frame.GroupCB, "BOTTOMLEFT", 0, -5)
	cfg.frame.TargetOnlyCB:Show()
	
	--CombatOnlyCB
	cfg.frame.CombatOnlyCB = CreateFrame("CheckButton", "CombatOnlyCB", cfg.frame, "InterfaceOptionsCheckButtonTemplate")
	cfg.frame.CombatOnlyCB:SetChecked(cfg.enableInCombatOnly)
	_G[cfg.frame.CombatOnlyCB:GetName().."Text"]:SetText("Enable in Combat only")
	cfg.frame.CombatOnlyCB:SetPoint("TOPLEFT", cfg.frame.TargetOnlyCB, "BOTTOMLEFT", 0, -5)
	cfg.frame.CombatOnlyCB:Show()
	
	
	
	InterfaceOptions_AddCategory(cfg.frame)
	
end

function SaveSettings(self)
	cfg.enableInRaid = cfg.frame.RaidCB:GetChecked()
	cfg.enableInSolo = cfg.frame.SoloCB:GetChecked()
	cfg.enableInGroup = cfg.frame.GroupCB:GetChecked()
	cfg.enableOnTargetOnly = cfg.frame.TargetOnlyCB:GetChecked()
	cfg.enableInCombatOnly = cfg.frame.CombatOnlyCB:GetChecked()
	IntN_Settings = cfg
end


local loader = CreateFrame("Frame")
loader:RegisterEvent("PLAYER_LOGIN")
loader:SetScript("OnEvent", function(self, event)
	if(event == "PLAYER_LOGIN") then
		self:UnregisterEvent("ADDON_LOADED")
		Setup()
	end
end)